@@ -20,14 +20,26 @@
 
 #include "Engine.h"
 
+#include <wibble/string.h>
+#include <wibble/regexp.h>
 #include <iostream>
 
 using namespace std;
+using namespace wibble;
 using namespace ept::apt;
 using namespace ept::debtags;
 
 Engine::Engine()
-	: m_filter_state(ANY), m_dirty(true), m_max(0) {}
+	: m_db(ept::axi::path_db()), m_stem("en"), m_filter_state(ANY), m_dirty(true), m_max(0)
+{
+	m_qp.set_default_op(Xapian::Query::OP_AND);
+        m_qp.set_database(m_db);
+        m_qp.set_stemmer(m_stem);
+        m_qp.set_stemming_strategy(Xapian::QueryParser::STEM_SOME);
+        m_qp.add_prefix("pkg", "XP");
+        m_qp.add_boolean_prefix("tag", "XT");
+        m_qp.add_boolean_prefix("sec", "XS");
+}
 
 struct EngineMatchDecider : public Xapian::MatchDecider
 {
@@ -56,11 +68,11 @@ struct EngineMatchDecider : public Xapian::MatchDecider
 
 static Xapian::Query allGames(Vocabulary& voc, const std::string& facet="game")
 {
-	set<Tag> games = voc.tags(facet);
+	set<std::string> games = voc.tags(facet);
 	vector<string> terms;
-	for (set<Tag>::const_iterator i = games.begin();
+	for (set<std::string>::const_iterator i = games.begin();
 			i != games.end(); ++i)
-		terms.push_back("XT" + i->fullname());
+		terms.push_back("XT" + *i);
 	return Xapian::Query(Xapian::Query::OP_OR, terms.begin(), terms.end());
 }
 
@@ -72,11 +84,33 @@ Xapian::Query Engine::makeQuery()
 	Xapian::Query ifacequery;
 
 	if (!m_filter_keywords.empty())
-		kwquery = m_textsearch.makePartialORQuery(m_filter_keywords);
-	if (m_filter_type.valid())
-		typequery = Xapian::Query("XT"+m_filter_type.fullname());
-	if (m_filter_iface.valid())
-		ifacequery = Xapian::Query("XT"+m_filter_iface.fullname());
+	{
+		// Add prefixes to tag names
+		Splitter splitter("[ \t]*,[ \t]*", REG_EXTENDED);
+		vector<string> kw;
+		for (Splitter::const_iterator i = splitter.begin(m_filter_keywords);
+				i != splitter.end(); ++i)
+		{
+			if (m_vocabulary.hasTag(*i))
+				kw.push_back("tag:" + *i);
+			else
+				kw.push_back(*i);
+		}
+		bool do_partial = not (kw.size() == 1 and kw[0].size() < 3);
+
+		kwquery = m_qp.parse_query(str::join(kw.begin(), kw.end(), " "),
+				Xapian::QueryParser::FLAG_BOOLEAN |
+				Xapian::QueryParser::FLAG_LOVEHATE |
+				Xapian::QueryParser::FLAG_BOOLEAN_ANY_CASE |
+				Xapian::QueryParser::FLAG_WILDCARD |
+				(do_partial ? Xapian::QueryParser::FLAG_PARTIAL : 0) |
+				Xapian::QueryParser::FLAG_PURE_NOT |
+				Xapian::QueryParser::FLAG_SPELLING_CORRECTION);
+	}
+	if (!m_filter_type.empty())
+		typequery = Xapian::Query("XT"+m_filter_type);
+	if (!m_filter_iface.empty())
+		ifacequery = Xapian::Query("XT"+m_filter_iface);
 		
 	if (kwquery.empty())
 		if (typequery.empty())
@@ -126,12 +160,12 @@ void Engine::recompute()
 	//cerr << "Engine recompute:" << endl;
 
 	// Compute the types
-	if (m_filter_type.valid())
+	if (!m_filter_type.empty())
 	{
 		//cerr << "  filter type: " << m_filter_type.fullname() << endl;
-		Tag tmp = m_filter_type;
-		m_filter_type = Tag();
-		Xapian::Enquire enquire(m_textsearch.db());
+		std::string tmp = m_filter_type;
+		m_filter_type = std::string();
+		Xapian::Enquire enquire(m_db);
 		enquire.set_query(makeQuery());
 
 		// Get all the results out of Xapian
@@ -144,10 +178,10 @@ void Engine::recompute()
 			for (Xapian::MSetIterator i = matches.begin(); i != matches.end(); ++i)
 			{
 				// Get all the game and interface tags in the result set
-				set<Tag> tags = m_debtags.getTagsOfItem(i.get_document().get_data());
-				for (set<Tag>::const_iterator j = tags.begin();
+				set<std::string> tags = m_debtags.getTagsOfItem(i.get_document().get_data());
+				for (set<std::string>::const_iterator j = tags.begin();
 						j != tags.end(); ++j)
-					if (j->facet().name() == mainFacet)
+					if (voc::getfacet(*j) == mainFacet)
 						m_types.insert(*j);
 			}
 		}
@@ -157,12 +191,12 @@ void Engine::recompute()
 	}
 
 	// Compute the interfaces
-	if (m_filter_iface.valid())
+	if (!m_filter_iface.empty())
 	{
 		//cerr << "  filter iface: " << m_filter_iface.fullname() << endl;
-		Tag tmp = m_filter_iface;
-		m_filter_iface = Tag();
-		Xapian::Enquire enquire(m_textsearch.db());
+		std::string tmp = m_filter_iface;
+		m_filter_iface = std::string();
+		Xapian::Enquire enquire(m_db);
 		enquire.set_query(makeQuery());
 
 		// Get all the results out of Xapian
@@ -175,10 +209,10 @@ void Engine::recompute()
 			for (Xapian::MSetIterator i = matches.begin(); i != matches.end(); ++i)
 			{
 				// Get all the game and interface tags in the result set
-				set<Tag> tags = m_debtags.getTagsOfItem(i.get_document().get_data());
-				for (set<Tag>::const_iterator j = tags.begin();
+				set<std::string> tags = m_debtags.getTagsOfItem(i.get_document().get_data());
+				for (set<std::string>::const_iterator j = tags.begin();
 						j != tags.end(); ++j)
-					if (j->facet().name() == secondaryFacet)
+					if (voc::getfacet(*j) == secondaryFacet)
 						m_interfaces.insert(*j);
 			}
 		}
@@ -187,7 +221,7 @@ void Engine::recompute()
 		//cerr << "  no filter iface" << endl;
 	}
 
-	Xapian::Enquire enquire(m_textsearch.db());
+	Xapian::Enquire enquire(m_db);
 	enquire.set_query(makeQuery());
 
 	//cerr << "  filter query: " << enquire.get_query().get_description() << endl;
@@ -221,22 +255,22 @@ void Engine::recompute()
 
 			// Get all the game and interface tags in the result set
 			// only for type or filter when they are not set
-			if (!m_filter_type.valid() || !m_filter_iface.valid())
+			if (m_filter_type.empty() || m_filter_iface.empty())
 			{
-				set<Tag> tags = m_debtags.getTagsOfItem(res.name);
-				for (set<Tag>::const_iterator j = tags.begin();
+				set<std::string> tags = m_debtags.getTagsOfItem(res.name);
+				for (set<std::string>::const_iterator j = tags.begin();
 						j != tags.end(); ++j)
-					if (!m_filter_type.valid() && j->facet().name() == mainFacet)
+					if (m_filter_type.empty() && voc::getfacet(*j) == mainFacet)
 						m_types.insert(*j);
-					else if (!m_filter_iface.valid() && j->facet().name() == secondaryFacet)
+					else if (m_filter_iface.empty() && voc::getfacet(*j) == secondaryFacet)
 						m_interfaces.insert(*j);
 			}
 		}
 	}
 	// Always keep the currently selected items in the lists
-	if (m_filter_type.valid())
+	if (!m_filter_type.empty())
 		m_types.insert(m_filter_type);
-	if (m_filter_iface.valid())
+	if (!m_filter_iface.empty())
 		m_interfaces.insert(m_filter_iface);
 
 
@@ -251,7 +285,7 @@ void Engine::recompute()
 
 std::vector<Result> Engine::related(const std::string& name, int count) const
 {
-	Xapian::Enquire enquire(m_textsearch.db());
+	Xapian::Enquire enquire(m_db);
 	
 	// Retrieve the document for the given package
 	enquire.set_query(Xapian::Query("XP"+name));
@@ -284,13 +318,13 @@ void Engine::setKeywordFilter(const std::string& keywords)
 	m_dirty = true;
 }
 
-void Engine::setTypeFilter(const ept::debtags::Tag& tag)
+void Engine::setTypeFilter(const std::string& tag)
 {
 	m_filter_type = tag;
 	m_dirty = true;
 }
 
-void Engine::setInterfaceFilter(const ept::debtags::Tag& tag)
+void Engine::setInterfaceFilter(const std::string& tag)
 {
 	m_filter_iface = tag;
 	m_dirty = true;
