@@ -23,7 +23,8 @@
 
 #include <ept/apt/apt.h>
 #include <ept/debtags/debtags.h>
-#include <ept/textsearch/textsearch.h>
+#include <ept/debtags/vocabulary.h>
+#include <ept/axi/axi.h>
 #include <ept/popcon/popcon.h>
 #include <string>
 #include <set>
@@ -58,22 +59,31 @@ protected:
 	/// Debtags data provider
 	ept::debtags::Debtags m_debtags;
 
+	/// Vocabulary data provider
+	ept::debtags::Vocabulary m_vocabulary;
+
 	/// Xapian data provider
-	ept::textsearch::TextSearch m_textsearch;
+	Xapian::Database m_db;
+
+	/// Xapian stemmer
+	Xapian::Stem m_stem;
+
+	/// Xapian query parser
+	Xapian::QueryParser m_qp;
 
 	/// Popcon scores
 	ept::popcon::Popcon m_popcon;
 
 	std::string m_filter_keywords;
-	ept::debtags::Tag m_filter_type;
-	ept::debtags::Tag m_filter_iface;
+	std::string m_filter_type;
+	std::string m_filter_iface;
 	Engine::State m_filter_state;
 
 	bool m_dirty;
 
 	std::vector<Result> m_results;
-	std::set<ept::debtags::Tag> m_types;
-	std::set<ept::debtags::Tag> m_interfaces;
+	std::set<std::string> m_types;
+	std::set<std::string> m_interfaces;
 
 	float m_max;
 	float m_res_max;
@@ -100,20 +110,20 @@ public:
 	ept::debtags::Debtags& debtags() { return m_debtags; }
 
 	/// Access the tag vocabulary
-	ept::debtags::Vocabulary& voc() { return m_debtags.vocabulary(); }
+	ept::debtags::Vocabulary& voc() { return m_vocabulary; }
 
 	/// Access the popcon data source
 	ept::popcon::Popcon& popcon() { return m_popcon; }
 
 	/// Get the list of available game types
-	const std::set<ept::debtags::Tag>& types()
+	const std::set<std::string>& types()
 	{
 		if (m_dirty) recompute();
 		return m_types;
 	}
 
 	/// Get the list of available interfaces
-	const std::set<ept::debtags::Tag>& interfaces()
+	const std::set<std::string>& interfaces()
 	{
 		if (m_dirty) recompute();
 		return m_interfaces;
@@ -144,12 +154,12 @@ public:
 	/**
 	 * Set the game type filter
 	 */
-	void setTypeFilter(const ept::debtags::Tag& tag = ept::debtags::Tag());
+	void setTypeFilter(const std::string& tag = std::string());
 
 	/**
 	 * Set the interface type filter
 	 */
-	void setInterfaceFilter(const ept::debtags::Tag& tag = ept::debtags::Tag());
+	void setInterfaceFilter(const std::string& tag = std::string());
 
 	/**
 	 * Set the installed state filter
