@@ -42,6 +42,7 @@
 #include <xapian.h>
 
 #include <iostream>
+#include <fstream>
 #include <cmath>
 
 #ifdef USE_GETTEXT
@@ -58,6 +59,8 @@
 
 #include <string.h>
 
+#include "libxdgutils/XDGDesktopEntry.h"
+
 namespace std {
 
 template<typename TAG, typename _Traits>
@@ -322,6 +325,55 @@ static void CallBackInstalledOrNot(Fl_Round_Button*, void *data)
 	UpdateUILists(ui);
 }
 
+static void StartPackage(const std::string pkgname)
+{
+	// find desktop file, fork, exec
+  
+	std::vector<std::string> desktoplist;
+
+	std::string pkglistfile =
+	  std::string("/var/lib/dpkg/info/") + pkgname + ".list";
+	std::ifstream pkglist(pkglistfile.c_str());
+	if (pkglist.is_open())
+	{
+		while ( pkglist.good() )
+		{
+			std::string line;
+			std::getline (pkglist,line);
+			if (0 == line.find("/usr/share/applications/")
+			    && line.npos != line.find(".desktop")) {
+				std::cout << line << endl;
+				desktoplist.push_back(line);
+			}
+		}
+		pkglist.close();
+	}
+	else
+		std::cout << "Unable to open file" << std::endl; 
+	if (1 == desktoplist.size()) {
+		std::string desktopfile(desktoplist.back());
+		std::cout << "Running " << desktopfile << endl;
+		XDGDesktopEntry entry(desktopfile.c_str());
+		entry.run("");
+	} else {
+		std::cout << "Not only one desktop file" << std::endl; 
+	}
+}
+
+static void CallBackScreenshotClick(Fl_Round_Button*, void *data)
+{
+	printf("CallBackScreenshotClick\n"); fflush(stdout);
+	GamesUI& ui = *static_cast<GamesUI*>(data);
+	PackageBrowser *ResultsBrowser = ui.ResultsBrowser;
+	std::string pkgname = ResultsBrowser->pkg_selected();
+	PackageRecord rec(ui.engine->apt().rawRecord(pkgname));
+
+	bool isinstalled = true; // FIXME find way to figure this out
+	if (isinstalled) {
+		StartPackage(pkgname);
+	}
+}
+
 #if 0
 static void OnResultsBrowserClick(Fl_Browser*, void *data)
 {
@@ -397,6 +449,12 @@ int main(int argc, const char* argv[])
 			engine.globalFilter = Xapian::Query(Xapian::Query::OP_OR,
 										Xapian::Query("XTrole::documentation"),
 										Xapian::Query("XTrole::program"));
+		} else if (wibble::str::endsWith(argv[0], "science") || opts.gowhere->stringValue() == "science") {
+			engine.mainFacet = "field";
+			engine.secondaryFacet = "interface";
+			engine.globalFilter = Xapian::Query(Xapian::Query::OP_OR,
+										Xapian::Query("XTrole::documentation"),
+										Xapian::Query("XTrole::program"));
 		} else {
 			engine.mainFacet = "game";
 			engine.secondaryFacet = "interface";
@@ -466,6 +524,7 @@ int main(int argc, const char* argv[])
 		mainui.InstalledOrNot->value(1); mainui.InstalledOrNot->redraw();
 		mainui.engine->setInstalledFilter(Engine::ANY);
 
+		mainui.ScreenshotWidget->callback((Fl_Callback*)CallBackScreenshotClick, &mainui);
 		mainui.Screenshot(new Fl_PNG_Image(FILE_NO_SCREENSHOT));
 		mainui.AboutView->load(HTMLDIR "/about.en.html");
 
