@@ -32,6 +32,8 @@
 #include <FL/Fl_Help_View.H>
 #include <FL/fl_draw.H>
 
+#include <string>
+
 //
 // Demonstrate how to derive a class extending Fl_Browser with interactively resizable columns
 // erco 1.10 12/09/2005
@@ -202,6 +204,7 @@ class PackageBrowser : public VersatileBrowser
 public:
 	PackageBrowser(int x, int y, int w, int h, const char *l = NULL);
 	virtual ~PackageBrowser();
+	const std::string pkg_selected(void) const;
 
 protected:
 	virtual void item_select(void *p, int s=1);
@@ -212,6 +215,7 @@ protected:
 private:
 	const unsigned int num_bar_images;
 	Fl_Image *bar_images[2*2];
+	std::string pkgname;
 };
 
 class PackageView : public Fl_Help_View
